==> ogre-1.12_1.12.2.orig.tar.gz: ogre-1.12.2/Samples/DeferredShading/src/SSAOLogic.cpp <==
/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2014 Torus Knot Software Ltd
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
same license as the rest of the engine.
-----------------------------------------------------------------------------
*/

#include "SSAOLogic.h"

#include "Ogre.h"
using namespace Ogre;

class ssaoListener: public Ogre::CompositorInstance::Listener
{
public:
    
    ssaoListener(Ogre::CompositorInstance* instance) : mInstance(instance) {}
   
    // this callback we will use to modify SSAO parameters
    void notifyMaterialRender(Ogre::uint32 pass_id, Ogre::MaterialPtr &mat)
    {
        if (pass_id != 42) // not SSAO, return
            return;

        // this is the camera you're using
        Ogre::Camera *cam = mInstance->getChain()->getViewport()->getCamera();

        // calculate the far-top-right corner in view-space
        Ogre::Vector3 farCorner = cam->getViewMatrix(true) * cam->getWorldSpaceCorners()[4];

        // get the pass
        Ogre::Pass *pass = mat->getBestTechnique()->getPass(0);

        // get the vertex shader parameters
        Ogre::GpuProgramParametersSharedPtr params = pass->getVertexProgramParameters();
        // set the camera's far-top-right corner
        if (params->_findNamedConstantDefinition("farCorner"))
            params->setNamedConstant("farCorner", farCorner);

        // get the fragment shader parameters
        params = pass->getFragmentProgramParameters();
        // set the projection matrix we need
        static const Ogre::Matrix4 CLIP_SPACE_TO_IMAGE_SPACE(
            0.5,    0,    0,  0.5,
            0,   -0.5,    0,  0.5,
            0,      0,    1,    0,
            0,      0,    0,    1);
        if (params->_findNamedConstantDefinition("ptMat"))
            params->setNamedConstant("ptMat", CLIP_SPACE_TO_IMAGE_SPACE * cam->getProjectionMatrixWithRSDepth());
        if (params->_findNamedConstantDefinition("far"))
            params->setNamedConstant("far", cam->getFarClipDistance());
    }
private:
    Ogre::CompositorInstance* mInstance;
};

Ogre::CompositorInstance::Listener* SSAOLogic::createListener(Ogre::CompositorInstance* instance)
{
    return new ssaoListener(instance);
}

==> ogre-14.4.1.tar.gz: ogre-14.4.1/Samples/DeferredShading/src/SSAOLogic.cpp <==
/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2014 Torus Knot Software Ltd
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
same license as the rest of the engine.
-----------------------------------------------------------------------------
*/

#include "SSAOLogic.h"

#include "Ogre.h"
using namespace Ogre;

class ssaoListener: public Ogre::CompositorInstance::Listener
{
public:
    
    ssaoListener(Ogre::CompositorInstance* instance) : mInstance(instance) {}
   
    // this callback we will use to modify SSAO parameters
    void notifyMaterialRender(Ogre::uint32 pass_id, Ogre::MaterialPtr &mat) override
    {
        if (pass_id != 42) // not SSAO, return
            return;

        // this is the camera you're using
        Ogre::Camera *cam = mInstance->getChain()->getViewport()->getCamera();

        // get the pass
        Ogre::Pass *pass = mat->getBestTechnique()->getPass(0);

        // get the fragment shader parameters
        auto params = pass->getFragmentProgramParameters();
        // set the projection matrix we need
        params->setNamedConstant("ptMat", Matrix4::CLIPSPACE2DTOIMAGESPACE * cam->getProjectionMatrixWithRSDepth());
    }
private:
    Ogre::CompositorInstance* mInstance;
};

Ogre::CompositorInstance::Listener* SSAOLogic::createListener(Ogre::CompositorInstance* instance)
{
    return new ssaoListener(instance);
}
